import io
import json
import shutil
import smtplib
import tomllib
from email.message import EmailMessage
from os import scandir
from pathlib import Path
import re

import requests
from requests.auth import HTTPBasicAuth


def main(end):
    with open("config.toml", "rb") as f:
        config = tomllib.load(f)

    with open("localisations.toml", "rb") as f:
        localisations = tomllib.load(f)

    while not end.is_set():
        metrics = {}

        # gather jobs
        completed_jobs = []
        error_jobs = []
        with scandir(config["UPLOAD_FOLDER"]) as uploads:
            for entry in uploads:
                if entry.is_dir():
                    with scandir(entry.path) as job:
                        for file in job:
                            if file.is_file():
                                if file.name == "done":
                                    completed_jobs.append(entry.path)
                                    break
                                elif file.name == "error":
                                    error_jobs.append(entry.path)
                                    break

        metrics["total_finished_jobs"] = len(completed_jobs)
        metrics["current_job_errors"] = len(error_jobs)

        try:
            local_hostname = config["MAIL"]["LOCAL_HOSTNAME"]
        except KeyError:
            local_hostname = None

        s = smtplib.SMTP(
            host=config["MAIL"]["SERVER"],
            port=config["MAIL"]["PORT"],
            local_hostname=local_hostname,
        )

        sent = 0
        with s:
            if len(completed_jobs) > 0:
                metrics["completed_job_languages"] = {}
                for job in completed_jobs:
                    sent += 1
                    with open(Path(job).joinpath("metadata.json")) as f:
                        metadata = json.load(f)

                    try:
                        metrics["completed_job_languages"][metadata["video_language"]] += 1
                    except KeyError:
                        metrics["completed_job_languages"][metadata["video_language"]] = 1

                    language = metadata["language"]

                    msg = EmailMessage()

                    if metadata["job_type"] == "subtitle":
                        msg["Subject"] = localisations["mail"]["subject"][language]
                        msg["From"] = config["MAIL"]["FROM"]
                        msg["To"] = metadata["email"]

                        msg.set_content(
                            localisations["mail"]["content"][language].format(metadata["filename"])
                        )

                        # filename.language.vtt
                        if metadata["video_language"] == "None":
                            filename = Path(metadata["filename"]).with_suffix(".vtt").name
                        else:
                            filename = (
                                Path(metadata["filename"])
                                .with_suffix(".{}.vtt".format(metadata["video_language"]))
                                .name
                            )

                        with open(Path(job).joinpath("audio.vtt")) as f:
                            msg.add_attachment(f.read(), filename=filename)

                    elif metadata["job_type"] == "transcript":
                        msg["Subject"] = localisations["mail"]["transcribe_subject"][language]
                        msg["From"] = config["MAIL"]["FROM"]
                        msg["To"] = metadata["email"]

                        msg.set_content(
                            localisations["mail"]["transcribe_content"][language].format(
                                metadata["filename"]
                            )
                        )

                        # filename.txt
                        filename = Path(metadata["filename"]).with_suffix(".txt").name

                        with open(Path(job).joinpath("audio.txt"), encoding="utf-8") as f:
                            transcript = f.read()

                        # reformat transcript
                        lines = []
                        for line in transcript.split("\n"):
                            match = re.fullmatch(transcript_pattern, line)

                            if match:
                                lines.append(match.groups())

                        transcript = ""
                        previous_speaker = None
                        for speaker, line in lines:
                            if previous_speaker == speaker:
                                transcript += f" {line}"
                            else:
                                previous_speaker = speaker
                                transcript += f"\n\n{speaker}: {line}"

                        msg.add_attachment(transcript, filename=filename)

                    s.send_message(msg)

                    shutil.rmtree(job)

            if len(error_jobs) > 0:
                try:
                    msg = EmailMessage()
                    msg["Subject"] = "Subtitle Service Error Report"
                    msg["From"] = config["MAIL"]["FROM"]
                    msg["To"] = config["MONITORING_MAIL"]

                    job_uuids = []
                    for job in error_jobs:
                        job_uuids.append(Path(job).name)

                    msg.set_content(
                        "The following jobs currently have errors:\n\n - {}".format(
                            "\n - ".join(job_uuids)
                        )
                    )

                    s.send_message(msg)
                    sent += 1

                except KeyError:
                    pass

        try:
            try:
                auth = HTTPBasicAuth(config["METRICS"]["USER"], config["METRICS"]["PASS"])
            except KeyError:
                auth = None

            r = requests.post(config["METRICS"]["URL"], json=metrics, auth=auth)

            try:
                r.raise_for_status()
            except requests.HTTPError as e:
                print("[MAILSERVICE] Could not send metrics to server: {}".format(e))

        except KeyError:
            pass

        print(
            "[MAILSERVICE] Sent {} mail(s). Sleeping for {} seconds.".format(
                sent, config["MAILSERVICE_INTERVAL"]
            )
        )

        end.wait(config["MAILSERVICE_INTERVAL"])


if __name__ == "__main__":
    import signal
    from threading import Event

    end = Event()

    transcript_pattern = re.compile(r"^\[(\w+)]: (.+)$")

    def handler(signum, frame):
        global end
        print(signum)
        end.set()

    signal.signal(signal.SIGINT, handler)
    signal.signal(signal.SIGTERM, handler)

    main(end)
