import json
from os import path, makedirs
from uuid import uuid4

import av
from flask import (
    current_app,
    render_template,
    abort,
    url_for,
    redirect,
    request,
    stream_with_context,
)

from forms import UploadForm


@current_app.route("/subtitle/<string:language>", methods=["GET", "POST"])
def subtitle(language: str):
    set_language(language)
    request.type = "subtitle"

    form = get_form(language)

    if form.is_submitted():
        if not form.email.data.endswith(current_app.config["MAIL_DOMAIN"]):
            form.email.data += current_app.config["MAIL_DOMAIN"]

        if form.validate():
            file = form.file.data
            video_language = "None" if form.language.data == "auto" else form.language.data
            metadata = {
                "job_type": "subtitle",
                "email": form.email.data,
                "language": language,
                "video_language": video_language,
                "filename": file.filename,
            }
            return stream_with_context(process_file(file, video_language, metadata, request.type))
        else:
            if form.email.data:
                form.email.data = form.email.data[: -len(current_app.config["MAIL_DOMAIN"])]

    if language != current_app.config["DEFAULT_LANGUAGE"]:
        form = language_fix_form(form, language)

    return render_template("upload.html", form=form)


@current_app.route("/transcribe/<string:language>", methods=["GET", "POST"])
def transcribe(language: str):
    set_language(language)
    request.type = "transcript"

    form = get_form(language)

    if form.is_submitted():
        if not form.email.data.endswith(current_app.config["MAIL_DOMAIN"]):
            form.email.data += current_app.config["MAIL_DOMAIN"]

        if form.validate():
            file = form.file.data
            video_language = "None" if form.language.data == "auto" else form.language.data
            metadata = {
                "job_type": "transcript",
                "email": form.email.data,
                "language": language,
                "video_language": video_language,
                "filename": file.filename,
            }
            return stream_with_context(process_file(file, video_language, metadata, request.type))
        else:
            if form.email.data:
                form.email.data = form.email.data[: -len(current_app.config["MAIL_DOMAIN"])]

    if language != current_app.config["DEFAULT_LANGUAGE"]:
        form = language_fix_form(form, language)

    return render_template("upload.html", form=form)


@current_app.route("/")
@current_app.route("/subtitle/")
def default():
    return redirect(url_for("subtitle", language=current_app.config["DEFAULT_LANGUAGE"]))


@current_app.route("/transcribe/")
def default_transcribe():
    return redirect(url_for("transcribe", language=current_app.config["DEFAULT_LANGUAGE"]))


def set_language(language):
    if language not in current_app.config["ENABLED_LOCALISATIONS"]:
        abort(404)

    request.language = language


def get_form(language):
    form = UploadForm()
    form.language.choices = [
        (k, v[language]) for k, v in current_app.config["LOCALISATIONS"]["language"]["options"].items()
    ]

    return form


def language_fix_form(form, language):
    form.email.label.text = current_app.config["LOCALISATIONS"]["email"]["label"][language]
    form.language.label.text = current_app.config["LOCALISATIONS"]["language"]["label"][language]
    form.file.label.text = current_app.config["LOCALISATIONS"]["file"]["label"][language]

    return form


def process_file(file, video_language, metadata, type):
    # provide immediate user feedback
    yield render_template("success.html")

    folder_name = str(uuid4())
    folder_path = path.join(current_app.config["UPLOAD_FOLDER"], folder_name)
    makedirs(folder_path)

    # Only save first audio stream
    with av.open(file.stream) as container:
        audio_stream = [stream for stream in container.streams if stream.type == "audio"][0]

        with av.open(path.join(folder_path, "audio.mkv"), "w") as out:
            out_stream = out.add_stream_from_template(audio_stream)

            for packet in container.demux(audio_stream):
                # Skip the "flushing" packets that `demux` generates.
                if packet.dts is None:
                    continue

                # Assign the packet to the new stream.
                packet.stream = out_stream

                out.mux(packet)

    with open(path.join(folder_path, "metadata.json"), "w") as f:
        json.dump(metadata, f)

    with open(path.join(folder_path, "video_language.txt"), "w") as f:
        f.write(video_language)

    if type == "subtitle":
        open(path.join(folder_path, "new"), "wb").close()
    elif type == "transcript":
        open(path.join(folder_path, "new_transcribe"), "wb").close()
    else:
        raise NotImplementedError

    # yield empty string to close response
    yield ""


@current_app.route("/health")
def health():
    return "", 204
